/****************************************************************************
** Meta object code from reading C++ file 'smb4kconfigpagebookmarks.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/smb4k/smb4kconfigpagebookmarks.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kconfigpagebookmarks.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KConfigPageBookmarks::qt_create_metaobjectdata<qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KConfigPageBookmarks",
        "bookmarksModified",
        "",
        "loadBookmarks",
        "saveBookmarks",
        "slotResetButtonClicked",
        "checked",
        "slotEditButtonClicked",
        "slotAddCategoryButtonClicked",
        "slotRemoveButtonClicked",
        "slotClearButtonClicked",
        "slotCurrentItemChanged",
        "QTreeWidgetItem*",
        "current",
        "previous",
        "slotItemSelectionChanged",
        "slotItemDoubleClicked",
        "item",
        "column",
        "slotLabelChanged",
        "text",
        "slotLabelEdited",
        "slotCategoryChanged",
        "slotCategoryEdited",
        "slotUserNameChanged",
        "slotUserNameEdited",
        "slotWorkgroupChanged",
        "slotWorkgroupEdited",
        "slotIpAddressChanged",
        "slotIpAddressEdited",
        "slotEnableButtons",
        "slotIconSizeChanged",
        "group"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'bookmarksModified'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadBookmarks'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveBookmarks'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResetButtonClicked'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotEditButtonClicked'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotAddCategoryButtonClicked'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotRemoveButtonClicked'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotClearButtonClicked'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotCurrentItemChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 12, 14 },
        }}),
        // Slot 'slotItemSelectionChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotItemDoubleClicked'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 17 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'slotLabelChanged'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotLabelEdited'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCategoryChanged'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotCategoryEdited'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUserNameChanged'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotUserNameEdited'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotWorkgroupChanged'
        QtMocHelpers::SlotData<void(const QString &)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotWorkgroupEdited'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotIpAddressChanged'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotIpAddressEdited'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnableButtons'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotIconSizeChanged'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KConfigPageBookmarks, qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KConfigPageBookmarks::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t>.metaTypes,
    nullptr
} };

void Smb4KConfigPageBookmarks::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KConfigPageBookmarks *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->bookmarksModified(); break;
        case 1: _t->loadBookmarks(); break;
        case 2: _t->saveBookmarks(); break;
        case 3: _t->slotResetButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->slotEditButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotAddCategoryButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->slotRemoveButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->slotClearButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->slotCurrentItemChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 9: _t->slotItemSelectionChanged(); break;
        case 10: _t->slotItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->slotLabelChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->slotLabelEdited(); break;
        case 13: _t->slotCategoryChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->slotCategoryEdited(); break;
        case 15: _t->slotUserNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->slotUserNameEdited(); break;
        case 17: _t->slotWorkgroupChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->slotWorkgroupEdited(); break;
        case 19: _t->slotIpAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->slotIpAddressEdited(); break;
        case 21: _t->slotEnableButtons(); break;
        case 22: _t->slotIconSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KConfigPageBookmarks::*)()>(_a, &Smb4KConfigPageBookmarks::bookmarksModified, 0))
            return;
    }
}

const QMetaObject *Smb4KConfigPageBookmarks::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KConfigPageBookmarks::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24Smb4KConfigPageBookmarksE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Smb4KConfigPageBookmarks::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void Smb4KConfigPageBookmarks::bookmarksModified()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
