/****************************************************************************
** Meta object code from reading C++ file 'smb4kbookmarkobject.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../smb4k-4.0.7/plasmoid/plugin/smb4kbookmarkobject.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kbookmarkobject.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19Smb4KBookmarkObjectE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KBookmarkObject::qt_create_metaobjectdata<qt_meta_tag_ZN19Smb4KBookmarkObjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KBookmarkObject",
        "changed",
        "",
        "icon",
        "QIcon",
        "workgroupName",
        "label",
        "url",
        "QUrl",
        "categoryName",
        "isCategory",
        "isMounted",
        "hostName",
        "shareName",
        "userName",
        "hostIpAddress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'icon'
        QtMocHelpers::MethodData<QIcon() const>(3, 2, QMC::AccessPublic, 0x80000000 | 4),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'workgroupName'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'categoryName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isCategory'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'isMounted'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'hostName'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'shareName'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'userName'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'hostIpAddress'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KBookmarkObject, qt_meta_tag_ZN19Smb4KBookmarkObjectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KBookmarkObject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Smb4KBookmarkObjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Smb4KBookmarkObjectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19Smb4KBookmarkObjectE_t>.metaTypes,
    nullptr
} };

void Smb4KBookmarkObject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KBookmarkObject *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: { QIcon _r = _t->icon();
            if (_a[0]) *reinterpret_cast<QIcon*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KBookmarkObject::*)()>(_a, &Smb4KBookmarkObject::changed, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->workgroupName(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 2: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->categoryName(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isCategory(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isMounted(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->hostName(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->shareName(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->userName(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->hostIpAddress(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWorkgroupName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 3: _t->setCategoryName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setCategory(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setMounted(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setUserName(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setHostIpAddress(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Smb4KBookmarkObject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KBookmarkObject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Smb4KBookmarkObjectE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Smb4KBookmarkObject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Smb4KBookmarkObject::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
