/****************************************************************************
** Meta object code from reading C++ file 'smb4kdeclarative.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../smb4k-4.0.7/plasmoid/plugin/smb4kdeclarative.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kdeclarative.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16Smb4KDeclarativeE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KDeclarative::qt_create_metaobjectdata<qt_meta_tag_ZN16Smb4KDeclarativeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KDeclarative",
        "workgroupsListChanged",
        "",
        "hostsListChanged",
        "sharesListChanged",
        "mountedSharesListChanged",
        "bookmarksListChanged",
        "profilesListChanged",
        "activeProfileChanged",
        "profileUsageChanged",
        "busy",
        "idle",
        "slotWorkgroupsListChanged",
        "slotHostsListChanged",
        "slotSharesListChanged",
        "slotMountedSharesListChanged",
        "slotBookmarksListChanged",
        "slotProfilesListChanged",
        "profiles",
        "slotActiveProfileChanged",
        "activeProfile",
        "slotProfileUsageChanged",
        "use",
        "slotCredentialsRequested",
        "NetworkItemPtr",
        "networkItem",
        "lookup",
        "Smb4KNetworkObject*",
        "object",
        "findNetworkItem",
        "QUrl",
        "url",
        "type",
        "openMountDialog",
        "mountShare",
        "mountBookmark",
        "Smb4KBookmarkObject*",
        "unmount",
        "unmountAll",
        "isShareMounted",
        "print",
        "addBookmark",
        "removeBookmark",
        "editBookmarks",
        "synchronize",
        "openCustomOptionsDialog",
        "startClient",
        "abortClient",
        "startMounter",
        "abortMounter",
        "preview",
        "openConfigurationDialog",
        "workgroups",
        "QQmlListProperty<Smb4KNetworkObject>",
        "hosts",
        "shares",
        "mountedShares",
        "bookmarks",
        "QQmlListProperty<Smb4KBookmarkObject>",
        "bookmarkCategories",
        "QQmlListProperty<Smb4KProfileObject>",
        "profileUsage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'workgroupsListChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hostsListChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sharesListChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mountedSharesListChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bookmarksListChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'profilesListChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeProfileChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'profileUsageChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'busy'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'idle'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWorkgroupsListChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHostsListChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSharesListChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMountedSharesListChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBookmarksListChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotProfilesListChanged'
        QtMocHelpers::SlotData<void(const QStringList &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QStringList, 18 },
        }}),
        // Slot 'slotActiveProfileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotProfileUsageChanged'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotCredentialsRequested'
        QtMocHelpers::SlotData<void(const NetworkItemPtr &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'lookup'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'lookup'
        QtMocHelpers::MethodData<void()>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'findNetworkItem'
        QtMocHelpers::MethodData<Smb4KNetworkObject *(const QUrl &, int)>(29, 2, QMC::AccessPublic, 0x80000000 | 27, {{
            { 0x80000000 | 30, 31 }, { QMetaType::Int, 32 },
        }}),
        // Method 'openMountDialog'
        QtMocHelpers::MethodData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'mountShare'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'mountBookmark'
        QtMocHelpers::MethodData<void(Smb4KBookmarkObject *)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 28 },
        }}),
        // Method 'unmount'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'unmountAll'
        QtMocHelpers::MethodData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'isShareMounted'
        QtMocHelpers::MethodData<bool(const QUrl &)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Method 'print'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'addBookmark'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'removeBookmark'
        QtMocHelpers::MethodData<void(Smb4KBookmarkObject *)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 28 },
        }}),
        // Method 'editBookmarks'
        QtMocHelpers::MethodData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'synchronize'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'openCustomOptionsDialog'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'startClient'
        QtMocHelpers::MethodData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'abortClient'
        QtMocHelpers::MethodData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'startMounter'
        QtMocHelpers::MethodData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'abortMounter'
        QtMocHelpers::MethodData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'preview'
        QtMocHelpers::MethodData<void(Smb4KNetworkObject *)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'openConfigurationDialog'
        QtMocHelpers::MethodData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'workgroups'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KNetworkObject>>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'hosts'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KNetworkObject>>(54, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'shares'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KNetworkObject>>(55, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'mountedShares'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KNetworkObject>>(56, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'bookmarks'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KBookmarkObject>>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'bookmarkCategories'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KBookmarkObject>>(59, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'profiles'
        QtMocHelpers::PropertyData<QQmlListProperty<Smb4KProfileObject>>(18, 0x80000000 | 60, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'activeProfile'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'profileUsage'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KDeclarative, qt_meta_tag_ZN16Smb4KDeclarativeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KDeclarative::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16Smb4KDeclarativeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16Smb4KDeclarativeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16Smb4KDeclarativeE_t>.metaTypes,
    nullptr
} };

void Smb4KDeclarative::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KDeclarative *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->workgroupsListChanged(); break;
        case 1: _t->hostsListChanged(); break;
        case 2: _t->sharesListChanged(); break;
        case 3: _t->mountedSharesListChanged(); break;
        case 4: _t->bookmarksListChanged(); break;
        case 5: _t->profilesListChanged(); break;
        case 6: _t->activeProfileChanged(); break;
        case 7: _t->profileUsageChanged(); break;
        case 8: _t->busy(); break;
        case 9: _t->idle(); break;
        case 10: _t->slotWorkgroupsListChanged(); break;
        case 11: _t->slotHostsListChanged(); break;
        case 12: _t->slotSharesListChanged(); break;
        case 13: _t->slotMountedSharesListChanged(); break;
        case 14: _t->slotBookmarksListChanged(); break;
        case 15: _t->slotProfilesListChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 16: _t->slotActiveProfileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->slotProfileUsageChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->slotCredentialsRequested((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1]))); break;
        case 19: _t->lookup((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 20: _t->lookup(); break;
        case 21: { Smb4KNetworkObject* _r = _t->findNetworkItem((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<Smb4KNetworkObject**>(_a[0]) = std::move(_r); }  break;
        case 22: _t->openMountDialog(); break;
        case 23: _t->mountShare((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 24: _t->mountBookmark((*reinterpret_cast<std::add_pointer_t<Smb4KBookmarkObject*>>(_a[1]))); break;
        case 25: _t->unmount((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 26: _t->unmountAll(); break;
        case 27: { bool _r = _t->isShareMounted((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->print((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 29: _t->addBookmark((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 30: _t->removeBookmark((*reinterpret_cast<std::add_pointer_t<Smb4KBookmarkObject*>>(_a[1]))); break;
        case 31: _t->editBookmarks(); break;
        case 32: _t->synchronize((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 33: _t->openCustomOptionsDialog((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 34: _t->startClient(); break;
        case 35: _t->abortClient(); break;
        case 36: _t->startMounter(); break;
        case 37: _t->abortMounter(); break;
        case 38: _t->preview((*reinterpret_cast<std::add_pointer_t<Smb4KNetworkObject*>>(_a[1]))); break;
        case 39: _t->openConfigurationDialog(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::workgroupsListChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::hostsListChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::sharesListChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::mountedSharesListChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::bookmarksListChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::profilesListChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::activeProfileChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::profileUsageChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::busy, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KDeclarative::*)()>(_a, &Smb4KDeclarative::idle, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlListProperty<Smb4KNetworkObject>*>(_v) = _t->workgroups(); break;
        case 1: *reinterpret_cast<QQmlListProperty<Smb4KNetworkObject>*>(_v) = _t->hosts(); break;
        case 2: *reinterpret_cast<QQmlListProperty<Smb4KNetworkObject>*>(_v) = _t->shares(); break;
        case 3: *reinterpret_cast<QQmlListProperty<Smb4KNetworkObject>*>(_v) = _t->mountedShares(); break;
        case 4: *reinterpret_cast<QQmlListProperty<Smb4KBookmarkObject>*>(_v) = _t->bookmarks(); break;
        case 5: *reinterpret_cast<QQmlListProperty<Smb4KBookmarkObject>*>(_v) = _t->bookmarkCategories(); break;
        case 6: *reinterpret_cast<QQmlListProperty<Smb4KProfileObject>*>(_v) = _t->profiles(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->activeProfile(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->profileUsage(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 7: _t->setActiveProfile(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Smb4KDeclarative::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KDeclarative::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16Smb4KDeclarativeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Smb4KDeclarative::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 40;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Smb4KDeclarative::workgroupsListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Smb4KDeclarative::hostsListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Smb4KDeclarative::sharesListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Smb4KDeclarative::mountedSharesListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Smb4KDeclarative::bookmarksListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Smb4KDeclarative::profilesListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Smb4KDeclarative::activeProfileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Smb4KDeclarative::profileUsageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Smb4KDeclarative::busy()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Smb4KDeclarative::idle()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
