/****************************************************************************
** Meta object code from reading C++ file 'smb4knetworksearchtoolbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/smb4k/smb4knetworksearchtoolbar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4knetworksearchtoolbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KNetworkSearchToolBar::qt_create_metaobjectdata<qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KNetworkSearchToolBar",
        "closeSearchBar",
        "",
        "search",
        "item",
        "abort",
        "jumpToResult",
        "url",
        "clearSearchResults",
        "slotReturnKeyPressed",
        "text",
        "slotSearchActionTriggered",
        "slotCloseButtonPressed",
        "slotDownActionTriggered",
        "slotUpActionTriggered",
        "slotClearSearch"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'closeSearchBar'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'search'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'abort'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'jumpToResult'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'clearSearchResults'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotReturnKeyPressed'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'slotSearchActionTriggered'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCloseButtonPressed'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDownActionTriggered'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpActionTriggered'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotClearSearch'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KNetworkSearchToolBar, qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KNetworkSearchToolBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolBar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t>.metaTypes,
    nullptr
} };

void Smb4KNetworkSearchToolBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KNetworkSearchToolBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->closeSearchBar(); break;
        case 1: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->abort(); break;
        case 3: _t->jumpToResult((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->clearSearchResults(); break;
        case 5: _t->slotReturnKeyPressed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotSearchActionTriggered(); break;
        case 7: _t->slotCloseButtonPressed(); break;
        case 8: _t->slotDownActionTriggered(); break;
        case 9: _t->slotUpActionTriggered(); break;
        case 10: _t->slotClearSearch(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KNetworkSearchToolBar::*)()>(_a, &Smb4KNetworkSearchToolBar::closeSearchBar, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KNetworkSearchToolBar::*)(const QString & )>(_a, &Smb4KNetworkSearchToolBar::search, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KNetworkSearchToolBar::*)()>(_a, &Smb4KNetworkSearchToolBar::abort, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KNetworkSearchToolBar::*)(const QString & )>(_a, &Smb4KNetworkSearchToolBar::jumpToResult, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KNetworkSearchToolBar::*)()>(_a, &Smb4KNetworkSearchToolBar::clearSearchResults, 4))
            return;
    }
}

const QMetaObject *Smb4KNetworkSearchToolBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KNetworkSearchToolBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25Smb4KNetworkSearchToolBarE_t>.strings))
        return static_cast<void*>(this);
    return QToolBar::qt_metacast(_clname);
}

int Smb4KNetworkSearchToolBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Smb4KNetworkSearchToolBar::closeSearchBar()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Smb4KNetworkSearchToolBar::search(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Smb4KNetworkSearchToolBar::abort()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Smb4KNetworkSearchToolBar::jumpToResult(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Smb4KNetworkSearchToolBar::clearSearchResults()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
