/****************************************************************************
** Meta object code from reading C++ file 'smb4kmounter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/core/smb4kmounter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kmounter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12Smb4KMounterE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KMounter::qt_create_metaobjectdata<qt_meta_tag_ZN12Smb4KMounterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KMounter",
        "updated",
        "",
        "SharePtr",
        "share",
        "mounted",
        "unmounted",
        "aboutToStart",
        "process",
        "finished",
        "mountedSharesListChanged",
        "requestCredentials",
        "slotStartJobs",
        "slotAboutToQuit",
        "slotOnlineStateChanged",
        "online",
        "slotAboutToChangeProfile",
        "slotActiveProfileChanged",
        "newProfile",
        "slotConfigChanged",
        "slotCredentialsUpdated",
        "QUrl",
        "url",
        "slotShareMounted",
        "mountPoint",
        "slotShareUnmounted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updated'
        QtMocHelpers::SignalData<void(const SharePtr &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'mounted'
        QtMocHelpers::SignalData<void(const SharePtr &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'unmounted'
        QtMocHelpers::SignalData<void(const SharePtr &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'aboutToStart'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'mountedSharesListChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCredentials'
        QtMocHelpers::SignalData<void(const SharePtr &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotStartJobs'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAboutToQuit'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOnlineStateChanged'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotAboutToChangeProfile'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotActiveProfileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'slotConfigChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCredentialsUpdated'
        QtMocHelpers::SlotData<void(const QUrl &)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotShareMounted'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Slot 'slotShareUnmounted'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KMounter, qt_meta_tag_ZN12Smb4KMounterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KMounter::staticMetaObject = { {
    QMetaObject::SuperData::link<KCompositeJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12Smb4KMounterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12Smb4KMounterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12Smb4KMounterE_t>.metaTypes,
    nullptr
} };

void Smb4KMounter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KMounter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updated((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 1: _t->mounted((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 2: _t->unmounted((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 3: _t->aboutToStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->mountedSharesListChanged(); break;
        case 6: _t->requestCredentials((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 7: _t->slotStartJobs(); break;
        case 8: _t->slotAboutToQuit(); break;
        case 9: _t->slotOnlineStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotAboutToChangeProfile(); break;
        case 11: _t->slotActiveProfileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->slotConfigChanged(); break;
        case 13: _t->slotCredentialsUpdated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 14: _t->slotShareMounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->slotShareUnmounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)(const SharePtr & )>(_a, &Smb4KMounter::updated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)(const SharePtr & )>(_a, &Smb4KMounter::mounted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)(const SharePtr & )>(_a, &Smb4KMounter::unmounted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)(int )>(_a, &Smb4KMounter::aboutToStart, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)(int )>(_a, &Smb4KMounter::finished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)()>(_a, &Smb4KMounter::mountedSharesListChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KMounter::*)(const SharePtr & )>(_a, &Smb4KMounter::requestCredentials, 6))
            return;
    }
}

const QMetaObject *Smb4KMounter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KMounter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12Smb4KMounterE_t>.strings))
        return static_cast<void*>(this);
    return KCompositeJob::qt_metacast(_clname);
}

int Smb4KMounter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KCompositeJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Smb4KMounter::updated(const SharePtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Smb4KMounter::mounted(const SharePtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Smb4KMounter::unmounted(const SharePtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Smb4KMounter::aboutToStart(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Smb4KMounter::finished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Smb4KMounter::mountedSharesListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Smb4KMounter::requestCredentials(const SharePtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
