/****************************************************************************
** Meta object code from reading C++ file 'smb4kmainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/smb4k/smb4kmainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kmainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15Smb4KMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN15Smb4KMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KMainWindow",
        "loadSettings",
        "",
        "saveSettings",
        "slotQuit",
        "slotConfigDialog",
        "slotAddBookmarks",
        "slotClientAboutToStart",
        "NetworkItemPtr",
        "item",
        "process",
        "slotClientFinished",
        "slotMounterAboutToStart",
        "slotMounterFinished",
        "slotVisualMountFeedback",
        "SharePtr",
        "share",
        "slotVisualUnmountFeedback",
        "slotSynchronizerAboutToStart",
        "dest",
        "slotSynchronizerFinished",
        "slotEndVisualFeedback",
        "slotEnableBookmarkAction",
        "slotNetworkBrowserVisibilityChanged",
        "visible",
        "slotSharesViewVisibilityChanged",
        "slotCredentialsRequested",
        "networkItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'loadSettings'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotQuit'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotConfigDialog'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddBookmarks'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotClientAboutToStart'
        QtMocHelpers::SlotData<void(const NetworkItemPtr &, int)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'slotClientFinished'
        QtMocHelpers::SlotData<void(const NetworkItemPtr &, int)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'slotMounterAboutToStart'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'slotMounterFinished'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'slotVisualMountFeedback'
        QtMocHelpers::SlotData<void(const SharePtr &)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotVisualUnmountFeedback'
        QtMocHelpers::SlotData<void(const SharePtr &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotSynchronizerAboutToStart'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'slotSynchronizerFinished'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'slotEndVisualFeedback'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnableBookmarkAction'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNetworkBrowserVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Slot 'slotSharesViewVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Slot 'slotCredentialsRequested'
        QtMocHelpers::SlotData<void(const NetworkItemPtr &)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KMainWindow, qt_meta_tag_ZN15Smb4KMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Smb4KMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Smb4KMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15Smb4KMainWindowE_t>.metaTypes,
    nullptr
} };

void Smb4KMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadSettings(); break;
        case 1: _t->saveSettings(); break;
        case 2: _t->slotQuit(); break;
        case 3: _t->slotConfigDialog(); break;
        case 4: _t->slotAddBookmarks(); break;
        case 5: _t->slotClientAboutToStart((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->slotClientFinished((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->slotMounterAboutToStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->slotMounterFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->slotVisualMountFeedback((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 10: _t->slotVisualUnmountFeedback((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 11: _t->slotSynchronizerAboutToStart((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->slotSynchronizerFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->slotEndVisualFeedback(); break;
        case 14: _t->slotEnableBookmarkAction(); break;
        case 15: _t->slotNetworkBrowserVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->slotSharesViewVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->slotCredentialsRequested((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Smb4KMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Smb4KMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int Smb4KMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
