/****************************************************************************
** Meta object code from reading C++ file 'smb4ksharesviewdockwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/smb4k/smb4ksharesviewdockwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4ksharesviewdockwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KSharesViewDockWidget::qt_create_metaobjectdata<qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KSharesViewDockWidget",
        "slotContextMenuRequested",
        "",
        "QPoint",
        "pos",
        "slotItemActivated",
        "QListWidgetItem*",
        "item",
        "slotItemSelectionChanged",
        "slotDropEvent",
        "Smb4KSharesViewItem*",
        "QDropEvent*",
        "e",
        "slotViewModeChanged",
        "QAction*",
        "action",
        "slotShareMounted",
        "SharePtr",
        "share",
        "slotShareUnmounted",
        "slotShareUpdated",
        "slotUnmountActionTriggered",
        "checked",
        "slotUnmountAllActionTriggered",
        "slotBookmarkActionTriggered",
        "slotAddCustomSettingsTriggered",
        "slotSynchronizeActionTriggered",
        "slotKonsoleActionTriggered",
        "slotFileManagerActionTriggered"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotItemActivated'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotItemSelectionChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDropEvent'
        QtMocHelpers::SlotData<void(Smb4KSharesViewItem *, QDropEvent *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 7 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotViewModeChanged'
        QtMocHelpers::SlotData<void(QAction *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotShareMounted'
        QtMocHelpers::SlotData<void(const SharePtr &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotShareUnmounted'
        QtMocHelpers::SlotData<void(const SharePtr &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotShareUpdated'
        QtMocHelpers::SlotData<void(const SharePtr &)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotUnmountActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotUnmountAllActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotBookmarkActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotAddCustomSettingsTriggered'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotSynchronizeActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotKonsoleActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotFileManagerActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KSharesViewDockWidget, qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KSharesViewDockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QDockWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t>.metaTypes,
    nullptr
} };

void Smb4KSharesViewDockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KSharesViewDockWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->slotItemActivated((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 2: _t->slotItemSelectionChanged(); break;
        case 3: _t->slotDropEvent((*reinterpret_cast<std::add_pointer_t<Smb4KSharesViewItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[2]))); break;
        case 4: _t->slotViewModeChanged((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 5: _t->slotShareMounted((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 6: _t->slotShareUnmounted((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 7: _t->slotShareUpdated((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 8: _t->slotUnmountActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotUnmountAllActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotBookmarkActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotAddCustomSettingsTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->slotSynchronizeActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotKonsoleActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->slotFileManagerActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *Smb4KSharesViewDockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KSharesViewDockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25Smb4KSharesViewDockWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QDockWidget::qt_metacast(_clname);
}

int Smb4KSharesViewDockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDockWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
