/****************************************************************************
** Meta object code from reading C++ file 'smb4kcustomsettingseditorwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/smb4k/smb4kcustomsettingseditorwidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kcustomsettingseditorwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KCustomSettingsEditorWidget::qt_create_metaobjectdata<qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KCustomSettingsEditorWidget",
        "edited",
        "",
        "changed",
        "slotIpAddressChanged",
        "text",
        "slotWorkgroupNameChanged",
        "slotAlwaysRemoutShareToggled",
        "checked",
        "slotUseWriteAccessToggled",
        "slotWriteAccessChanged",
        "index",
        "slotCifsUnixExtensionSupportToggled",
        "slotUseIdsToggled",
        "slotUseFileModeToggled",
        "slotFileModeChanged",
        "slotUseDirectoryModeToggled",
        "slotDirectoryModeChanged",
        "slotUseSecurityModeToggled",
        "slotSecurityModeChanged",
        "slotUseClientProtocolVersionsToggled",
        "slotMinimalClientProtocolVersionChanged",
        "slotMaximalClientProtocolVersionChanged",
        "slotUseRemoteSmbPortToggled",
        "slotRemoteSmbPortChanged",
        "port",
        "slotUseKerberosToggled",
        "slotFindMacAddressClicked",
        "slotMacAddressChanged",
        "slotSendPacketBeforeScanToggled",
        "slotSendPacketBeforeMountToggled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'edited'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'slotIpAddressChanged'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotWorkgroupNameChanged'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotAlwaysRemoutShareToggled'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotUseWriteAccessToggled'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotWriteAccessChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotCifsUnixExtensionSupportToggled'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotUseIdsToggled'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotUseFileModeToggled'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotFileModeChanged'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotUseDirectoryModeToggled'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotDirectoryModeChanged'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotUseSecurityModeToggled'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotSecurityModeChanged'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotUseClientProtocolVersionsToggled'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotMinimalClientProtocolVersionChanged'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotMaximalClientProtocolVersionChanged'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotUseRemoteSmbPortToggled'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotRemoteSmbPortChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'slotUseKerberosToggled'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotFindMacAddressClicked'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotMacAddressChanged'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotSendPacketBeforeScanToggled'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotSendPacketBeforeMountToggled'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KCustomSettingsEditorWidget, qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KCustomSettingsEditorWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t>.metaTypes,
    nullptr
} };

void Smb4KCustomSettingsEditorWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KCustomSettingsEditorWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->edited((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->slotIpAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->slotWorkgroupNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->slotAlwaysRemoutShareToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->slotUseWriteAccessToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotWriteAccessChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotCifsUnixExtensionSupportToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->slotUseIdsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->slotUseFileModeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotFileModeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->slotUseDirectoryModeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotDirectoryModeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->slotUseSecurityModeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotSecurityModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->slotUseClientProtocolVersionsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->slotMinimalClientProtocolVersionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotMaximalClientProtocolVersionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->slotUseRemoteSmbPortToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->slotRemoteSmbPortChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->slotUseKerberosToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->slotFindMacAddressClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->slotMacAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->slotSendPacketBeforeScanToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->slotSendPacketBeforeMountToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KCustomSettingsEditorWidget::*)(bool )>(_a, &Smb4KCustomSettingsEditorWidget::edited, 0))
            return;
    }
}

const QMetaObject *Smb4KCustomSettingsEditorWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KCustomSettingsEditorWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31Smb4KCustomSettingsEditorWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int Smb4KCustomSettingsEditorWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void Smb4KCustomSettingsEditorWidget::edited(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
