/****************************************************************************
** Meta object code from reading C++ file 'smb4knetworkbrowserdockwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/smb4k/smb4knetworkbrowserdockwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4knetworkbrowserdockwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KNetworkBrowserDockWidget::qt_create_metaobjectdata<qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KNetworkBrowserDockWidget",
        "slotContextMenuRequested",
        "",
        "QPoint",
        "pos",
        "slotItemActivated",
        "QTreeWidgetItem*",
        "item",
        "column",
        "slotItemSelectionChanged",
        "slotClientAboutToStart",
        "NetworkItemPtr",
        "process",
        "slotClientFinished",
        "slotWorkgroups",
        "slotWorkgroupMembers",
        "WorkgroupPtr",
        "workgroup",
        "slotShares",
        "HostPtr",
        "host",
        "slotRescanAbortActionTriggered",
        "checked",
        "slotAddBookmark",
        "slotMountManually",
        "slotAuthentication",
        "slotAddCustomSettings",
        "slotPreview",
        "slotPrint",
        "slotMountActionTriggered",
        "slotMountActionChanged",
        "active",
        "slotShareMounted",
        "SharePtr",
        "share",
        "slotShareUnmounted",
        "slotMounterAboutToStart",
        "slotMounterFinished",
        "slotShowSearchToolBar",
        "slotHideSearchToolBar",
        "slotPerformSearch",
        "slotStopSearch",
        "slotSearchResults",
        "QList<SharePtr>",
        "shares",
        "slotJumpToResult",
        "url",
        "slotClearSearchResults"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotItemActivated'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'slotItemSelectionChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotClientAboutToStart'
        QtMocHelpers::SlotData<void(const NetworkItemPtr &, int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 7 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotClientFinished'
        QtMocHelpers::SlotData<void(const NetworkItemPtr &, int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 7 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotWorkgroups'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotWorkgroupMembers'
        QtMocHelpers::SlotData<void(const WorkgroupPtr &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotShares'
        QtMocHelpers::SlotData<void(const HostPtr &)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotRescanAbortActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotAddBookmark'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotMountManually'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotAuthentication'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotAddCustomSettings'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotPreview'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotPrint'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotMountActionTriggered'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotMountActionChanged'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'slotShareMounted'
        QtMocHelpers::SlotData<void(const SharePtr &)>(32, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'slotShareUnmounted'
        QtMocHelpers::SlotData<void(const SharePtr &)>(35, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'slotMounterAboutToStart'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotMounterFinished'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotShowSearchToolBar'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHideSearchToolBar'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPerformSearch'
        QtMocHelpers::SlotData<void(const QString &)>(40, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'slotStopSearch'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSearchResults'
        QtMocHelpers::SlotData<void(const QList<SharePtr> &)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'slotJumpToResult'
        QtMocHelpers::SlotData<void(const QString &)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 46 },
        }}),
        // Slot 'slotClearSearchResults'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KNetworkBrowserDockWidget, qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KNetworkBrowserDockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QDockWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t>.metaTypes,
    nullptr
} };

void Smb4KNetworkBrowserDockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KNetworkBrowserDockWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->slotItemActivated((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotItemSelectionChanged(); break;
        case 3: _t->slotClientAboutToStart((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->slotClientFinished((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->slotWorkgroups(); break;
        case 6: _t->slotWorkgroupMembers((*reinterpret_cast<std::add_pointer_t<WorkgroupPtr>>(_a[1]))); break;
        case 7: _t->slotShares((*reinterpret_cast<std::add_pointer_t<HostPtr>>(_a[1]))); break;
        case 8: _t->slotRescanAbortActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotAddBookmark((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotMountManually((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotAuthentication((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->slotAddCustomSettings((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotPreview((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->slotPrint((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->slotMountActionTriggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->slotMountActionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->slotShareMounted((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 18: _t->slotShareUnmounted((*reinterpret_cast<std::add_pointer_t<SharePtr>>(_a[1]))); break;
        case 19: _t->slotMounterAboutToStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotMounterFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->slotShowSearchToolBar(); break;
        case 22: _t->slotHideSearchToolBar(); break;
        case 23: _t->slotPerformSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->slotStopSearch(); break;
        case 25: _t->slotSearchResults((*reinterpret_cast<std::add_pointer_t<QList<SharePtr>>>(_a[1]))); break;
        case 26: _t->slotJumpToResult((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->slotClearSearchResults(); break;
        default: ;
        }
    }
}

const QMetaObject *Smb4KNetworkBrowserDockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KNetworkBrowserDockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29Smb4KNetworkBrowserDockWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QDockWidget::qt_metacast(_clname);
}

int Smb4KNetworkBrowserDockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDockWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 28;
    }
    return _id;
}
QT_WARNING_POP
