/****************************************************************************
** Meta object code from reading C++ file 'smb4kclient.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../smb4k-4.0.7/core/smb4kclient.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4kclient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11Smb4KClientE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KClient::qt_create_metaobjectdata<qt_meta_tag_ZN11Smb4KClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KClient",
        "aboutToStart",
        "",
        "NetworkItemPtr",
        "item",
        "type",
        "finished",
        "workgroups",
        "hosts",
        "WorkgroupPtr",
        "workgroup",
        "shares",
        "HostPtr",
        "host",
        "files",
        "QList<FilePtr>",
        "list",
        "searchResults",
        "QList<SharePtr>",
        "requestCredentials",
        "networkItem",
        "slotStartJobs",
        "slotOnlineStateChanged",
        "online",
        "slotResult",
        "KJob*",
        "job",
        "slotAboutToQuit",
        "slotAbort",
        "slotCredentialsUpdated",
        "QUrl",
        "url"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToStart'
        QtMocHelpers::SignalData<void(const NetworkItemPtr &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(const NetworkItemPtr &, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'workgroups'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hosts'
        QtMocHelpers::SignalData<void(const WorkgroupPtr &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'shares'
        QtMocHelpers::SignalData<void(const HostPtr &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'files'
        QtMocHelpers::SignalData<void(const QList<FilePtr> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'searchResults'
        QtMocHelpers::SignalData<void(const QList<SharePtr> &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 16 },
        }}),
        // Signal 'requestCredentials'
        QtMocHelpers::SignalData<void(const NetworkItemPtr &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 20 },
        }}),
        // Slot 'slotStartJobs'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOnlineStateChanged'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'slotResult'
        QtMocHelpers::SlotData<void(KJob *)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotAboutToQuit'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAbort'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCredentialsUpdated'
        QtMocHelpers::SlotData<void(const QUrl &)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Smb4KClient, qt_meta_tag_ZN11Smb4KClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KClient::staticMetaObject = { {
    QMetaObject::SuperData::link<KCompositeJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11Smb4KClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11Smb4KClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11Smb4KClientE_t>.metaTypes,
    nullptr
} };

void Smb4KClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToStart((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->workgroups(); break;
        case 3: _t->hosts((*reinterpret_cast<std::add_pointer_t<WorkgroupPtr>>(_a[1]))); break;
        case 4: _t->shares((*reinterpret_cast<std::add_pointer_t<HostPtr>>(_a[1]))); break;
        case 5: _t->files((*reinterpret_cast<std::add_pointer_t<QList<FilePtr>>>(_a[1]))); break;
        case 6: _t->searchResults((*reinterpret_cast<std::add_pointer_t<QList<SharePtr>>>(_a[1]))); break;
        case 7: _t->requestCredentials((*reinterpret_cast<std::add_pointer_t<NetworkItemPtr>>(_a[1]))); break;
        case 8: _t->slotStartJobs(); break;
        case 9: _t->slotOnlineStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 11: _t->slotAboutToQuit(); break;
        case 12: _t->slotAbort(); break;
        case 13: _t->slotCredentialsUpdated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const NetworkItemPtr & , int )>(_a, &Smb4KClient::aboutToStart, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const NetworkItemPtr & , int )>(_a, &Smb4KClient::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)()>(_a, &Smb4KClient::workgroups, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const WorkgroupPtr & )>(_a, &Smb4KClient::hosts, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const HostPtr & )>(_a, &Smb4KClient::shares, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const QList<FilePtr> & )>(_a, &Smb4KClient::files, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const QList<SharePtr> & )>(_a, &Smb4KClient::searchResults, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Smb4KClient::*)(const NetworkItemPtr & )>(_a, &Smb4KClient::requestCredentials, 7))
            return;
    }
}

const QMetaObject *Smb4KClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11Smb4KClientE_t>.strings))
        return static_cast<void*>(this);
    return KCompositeJob::qt_metacast(_clname);
}

int Smb4KClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KCompositeJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void Smb4KClient::aboutToStart(const NetworkItemPtr & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Smb4KClient::finished(const NetworkItemPtr & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Smb4KClient::workgroups()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Smb4KClient::hosts(const WorkgroupPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Smb4KClient::shares(const HostPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Smb4KClient::files(const QList<FilePtr> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Smb4KClient::searchResults(const QList<SharePtr> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Smb4KClient::requestCredentials(const NetworkItemPtr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
