/****************************************************************************
** Meta object code from reading C++ file 'smb4knetworkobject.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../smb4k-4.0.7/plasmoid/plugin/smb4knetworkobject.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smb4knetworkobject.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18Smb4KNetworkObjectE_t {};
} // unnamed namespace

template <> constexpr inline auto Smb4KNetworkObject::qt_create_metaobjectdata<qt_meta_tag_ZN18Smb4KNetworkObjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Smb4KNetworkObject",
        "changed",
        "",
        "icon",
        "QIcon",
        "type",
        "NetworkItem",
        "parentType",
        "workgroupName",
        "hostName",
        "shareName",
        "name",
        "comment",
        "url",
        "QUrl",
        "parentUrl",
        "isMounted",
        "isPrinter",
        "mountpoint",
        "isMasterBrowser",
        "isInaccessible",
        "Network",
        "Workgroup",
        "Host",
        "Share",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'icon'
        QtMocHelpers::MethodData<QIcon() const>(3, 2, QMC::AccessPublic, 0x80000000 | 4),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<enum NetworkItem>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'parentType'
        QtMocHelpers::PropertyData<enum NetworkItem>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'workgroupName'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'hostName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'shareName'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'comment'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'parentUrl'
        QtMocHelpers::PropertyData<QUrl>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'isMounted'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'isPrinter'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'mountpoint'
        QtMocHelpers::PropertyData<QUrl>(18, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'isMasterBrowser'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'isInaccessible'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'NetworkItem'
        QtMocHelpers::EnumData<enum NetworkItem>(6, 6, QMC::EnumFlags{}).add({
            {   21, NetworkItem::Network },
            {   22, NetworkItem::Workgroup },
            {   23, NetworkItem::Host },
            {   24, NetworkItem::Share },
            {   25, NetworkItem::Unknown },
        }),
    };
    return QtMocHelpers::metaObjectData<Smb4KNetworkObject, qt_meta_tag_ZN18Smb4KNetworkObjectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Smb4KNetworkObject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18Smb4KNetworkObjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18Smb4KNetworkObjectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18Smb4KNetworkObjectE_t>.metaTypes,
    nullptr
} };

void Smb4KNetworkObject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Smb4KNetworkObject *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: { QIcon _r = _t->icon();
            if (_a[0]) *reinterpret_cast<QIcon*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Smb4KNetworkObject::*)()>(_a, &Smb4KNetworkObject::changed, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum NetworkItem*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<enum NetworkItem*>(_v) = _t->parentType(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->workgroupName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->hostName(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->shareName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->comment(); break;
        case 7: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 8: *reinterpret_cast<QUrl*>(_v) = _t->parentUrl(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isMounted(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isPrinter(); break;
        case 11: *reinterpret_cast<QUrl*>(_v) = _t->mountpoint(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isMasterBrowser(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isInaccessible(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setType(*reinterpret_cast<enum NetworkItem*>(_v)); break;
        case 2: _t->setWorkgroupName(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setComment(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 9: _t->setMounted(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setPrinter(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setMountpoint(*reinterpret_cast<QUrl*>(_v)); break;
        case 12: _t->setMasterBrowser(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setInaccessible(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Smb4KNetworkObject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Smb4KNetworkObject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18Smb4KNetworkObjectE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Smb4KNetworkObject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void Smb4KNetworkObject::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
